
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#include <cstdarg>
#include <cstdio>		// snprintf
#include <cstring>		// strcpy

#include "Log.hh"


namespace Mf {


static LogLevel logLevel_ = WARNING;

inline void printLog_(int logLevel, const char* fmt, va_list args)
{
	if (logLevel_ < logLevel) return;

	switch (logLevel)
	{
		case ERROR:
			fprintf(stderr, "  error: ");
			break;
		case WARNING:
			fprintf(stderr, "warning: ");
			break;
		case INFO:
			fprintf(stderr, "   info: ");
			break;
		case DEBUGGING:
			fprintf(stderr, "  debug: ");
			break;
	}

	vfprintf(stderr, fmt, args);
	fprintf(stderr, "\n");
}


LogLevel setLogLevel(LogLevel level)
{
	if (level != 0)
		logLevel_ = level;

	return logLevel_;
}


void logError(const char* fmt, ...)
{
	va_list args;
	va_start(args, fmt);

	printLog_(ERROR, fmt, args);

	va_end(args);
}

void logWarning(const char* fmt, ...)
{
	va_list args;
	va_start(args, fmt);

	printLog_(WARNING, fmt, args);

	va_end(args);
}

void logInfo(const char* fmt, ...)
{
	va_list args;
	va_start(args, fmt);

	printLog_(INFO, fmt, args);

	va_end(args);
}

void logDebug(const char* fmt, ...)
{
	va_list args;
	va_start(args, fmt);

	printLog_(DEBUGGING, fmt, args);

	va_end(args);
}


} // namespace Mf

